#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <syslog.h>

#include <sys/types.h>
#include <limits.h>
#include <ndbm.h>
#include <sysexits.h>
#include <fcntl.h>
#include <time.h>
#include <ctype.h>

#include "Config.h"

#ifdef DBF_1
static char lmt_name[] = LMTNAME;
#endif

char *pname;
char *user;			/* user to operate */

DBM *db = NULL;
datum key, val;
int listonly = 0;

void usage()
{
#ifndef DBF_1

	fprintf(stderr, "\
Usage: %s a|c|m|d|l user [time]\n\
	a <user> <time> - assign <time> minutes for user <user>\n\
	c <user> <time> - create new user <user> with <time> minutes\n\
	m <user> <time> - add/sub <time> minutes for user <user>\n\
	d <user>        - delete user <user>\n\
	l <user>        - list <user> ('-' for list all)\n\
", pname);

#else

    fprintf(stderr, "\
Usage: %s a[z]|c[z]|s[z]|m[z]|d|l[z]|t [parm]\n\
 a[z] <user> <time_lim> <time_lim_left> <day_lim> <day_lim_left> [<tot_used>]\n\
                - assign time minutes for user <user>\n\
 c[z] <user> <time_lim> <time_lim_left> <day_lim> <day_lim_left> [<tot_used>]\n\
                - create new user <user> with time minutes\n\
 s[z] <user> <time_lim> <time_lim_left> <day_lim> <day_lim_left> [<tot_used>]\n\
                - set time minutes for user <user>\n\
 m[z] <user> <time_lim> <time_lim_left> <day_lim> <day_lim_left> [<tot_used>]\n\
                - add/sub time minutes for user <user>\n\
 d <user>       - delete user <user>\n\
 l[z] <user>    - list <user> ('-' for list all)\n\
 t [time stamp] - set last modify 'day limits' table\n\
", pname);

#endif
    exit(EX_USAGE);
}

void openbase(void)
{
    if(db)
      return;
    db = dbm_open(DBFILE, (listonly ? O_RDONLY : O_RDWR) | O_EXLOCK, 0600);
    if (!db) {
	perror("Unable to open database file");
	exit(EX_OSFILE);
    }
}

int adduser(u, create,
#ifdef DBF_1
              tl, tll, dl, dll, tu, tuflag)
char u[];
int create;
long tl;
long tll;
long dl;
long dll;
long tu;
int tuflag;
#else
              tq)
char u[];
int create;
time_t tq;
#endif
{
#ifdef DBF_1
    TQ_st tq;
#endif

    openbase();

    if(create == 'c'
#ifdef DBF_1
       || create == 's' || tuflag < 0
#endif    
      )
     {
      key.dptr = u;
      key.dsize = strlen(u) + 1;
      val = dbm_fetch(db, key);
#ifdef DBF_1
      if (create == 'c') {
#endif
        if (val.dptr) {
     	  printf("User `%s' already exists.\n", u);
	  exit(EX_NOUSER);
        }
#ifdef DBF_1        
      } else if (create == 's') {
        if (!val.dptr) {
   	  printf("User `%s' not found.\n", u);
	  exit(EX_NOUSER);
        }
      }
#endif      
     }
     
#ifdef DBF_1     
    tq.tl = tl;
    tq.tll = tll;
    tq.dl = dl;
    tq.dll = dll;
    tq.tot = (
              (!tuflag || (tuflag < 0 && !val.dptr)) ? ((tl >= 0) ? (tl - tll) : (0 - tll)) : 
              ((tuflag > 0) ? tu : ((TQ_st *)val.dptr)->tot)
             );
#endif
    
    key.dptr = u;
    key.dsize = strlen(u) + 1;
    val.dptr = (void *) &tq;
    val.dsize = sizeof(tq);
    return dbm_store(db, key, val, DBM_REPLACE);
}

int deluser(u)
char u[];
{
    openbase();
    
    key.dptr = u;
    key.dsize = strlen(u) + 1;
    return dbm_delete(db, key);
}

#ifdef DBF_1

static int ph = 1;

int listprint(u, tq)
char u[];
TQ_st *tq;
{
    
    if (!strcmp(u, lmt_name))
	return 0;
	
    if (ph > 0) {
	ph = -1;
	printf("---user---     --total time/left--   --day limit/left--   --total online time--\n");
    }
    printf(" %-8s       %7ld   %7ld      %6ld   %6ld       %16ld\n",
	   u, tq->tl / 60, tq->tll / 60, tq->dl / 60, tq->dll / 60, tq->tot / 60);
    return 0;
}


int lmtput(void)
{
    char buf[80];

    if(!ph)
      return 0;

    printf("Last modify 'day limits' table: ");
    key.dptr = lmt_name;
    key.dsize = strlen(lmt_name) + 1;
    val = dbm_fetch(db, key);
    if (!val.dptr)
	printf("undefined\n");
    else {
	strftime(buf, sizeof(buf), "%Y/%m/%d %H:%M:%S", localtime((time_t *) val.dptr));
	printf("%s\n", buf);
    }
    return (EX_NOUSER);
}
#endif

int listuser(u)
char u[];
{
#ifdef DBF_1
    TQ_st *tq;
#endif

    openbase();
    
    if (strcmp(u, "-")) {
	key.dptr = u;
	key.dsize = strlen(u) + 1;
	val = dbm_fetch(db, key);
	if (val.dptr) {
#ifdef DBF_1
	    tq = (TQ_st *) val.dptr;
	    listprint(u, tq);
#else	    
            printf("*** User `%s', time left %ld minutes\n",
      	           u, *(time_t *)val.dptr / 60);
#endif	    
	} else {
	    printf("*** Time limit not set for user `%s'\n", u);
	    exit(EX_NOUSER);
	}
    } else {
	key = dbm_firstkey(db);
	if (key.dptr)
	    do {
		val = dbm_fetch(db, key);
#ifdef DBF_1
		tq = (TQ_st *) val.dptr;
		listprint(key.dptr, tq);
#else
                printf("%-10s %6ld\n", key.dptr, *(time_t *)val.dptr / 60);
#endif
		key = dbm_nextkey(db);
	    } while (key.dptr);
    }
#ifdef DBF_1
    lmtput();
#endif    
    return 0;
}

#ifdef DBF_1
int lmtset(u)
char u[];
{
    time_t t;
    struct tm tm;

    openbase();

    if (u) {
	sscanf(u, "%d/%d/%d %d:%d:%d", &tm.tm_year, &tm.tm_mon, &tm.tm_mday, &tm.tm_hour, &tm.tm_min, &tm.tm_sec);
	tm.tm_year -= 1900;
	tm.tm_mon--;
	tm.tm_isdst = -1;
	t = mktime(&tm);
    } else
	time(&t);
    key.dptr = lmt_name;
    key.dsize = strlen(lmt_name) + 1;
    val.dptr = (void *) &t;
    val.dsize = sizeof(t);
    return dbm_store(db, key, val, DBM_REPLACE);
}
#endif

int moduser(u, 
#ifdef DBF_1
               tl, tll, dl, dll, tu, tuflag)
char u[];
long tl;
long tll;
long dl;
long dll;
long tu;
int tuflag;
#else
               tq)
               
char u[];
time_t tq;
#endif
{
#ifdef DBF_1
    TQ_st tq;
    TQ_st *tqp;
#endif

    openbase();

    key.dptr = u;
    key.dsize = strlen(u) + 1;
    val = dbm_fetch(db, key);
    if (!val.dsize) {
	printf("User `%s' not found\n", u);
	exit(EX_NOUSER);
    }
#ifdef DBF_1    
    tqp = (TQ_st *) val.dptr;
    tq.tl  = tqp->tl  + tl;
    tq.tll = tqp->tll + tll;
    tq.dl  = tqp->dl  + dl;
    tq.dll = tqp->dll + dll;
    tq.tot = (
              (tuflag <= 0) ? 
              ((tuflag)  ?  tqp->tot : ((tq.tl >= 0) ? (tq.tl - tq.tll) : (0 - tq.tll))) : 
              tqp->tot + tu
             );
#else
    tq += *(time_t *)val.dptr;
#endif    
    val.dptr = (void *) &tq;
    val.dsize = sizeof(tq);
    return dbm_store(db, key, val, DBM_REPLACE);
}

#ifdef DBF_1
int renew_day_limit(void)
{
    DBM *dbl;
    TQ_st tq;
    datum key_l, val_l;
    char u[17];

    dbl = dbm_open(DBFILE, O_RDONLY, 0600);
    if (!dbl) {
	perror("Unable to open database file (renew)");
	exit(EX_OSFILE);
    }

    key_l = dbm_firstkey(dbl);
    if (key_l.dptr)
	do {
	    val_l = dbm_fetch(dbl, key_l);
	    if (strcmp(key_l.dptr, lmt_name)) {
	      strncpy(u, key_l.dptr, key_l.dsize);
	      memcpy(&tq, val_l.dptr, sizeof(tq));
	      tq.dll = tq.dl;
	      key.dptr = u;
	      key.dsize = strlen(u) + 1;
	      val.dptr = (void *) &tq;
	      val.dsize = sizeof(tq);
	      dbm_store(db, key, val, DBM_REPLACE);
	    }
	    key_l = dbm_nextkey(dbl);
	} while (key_l.dptr);
    dbm_close(dbl);
    return 0;
}

#endif

int main(ac, av)
char *av[];
{
#ifdef DBF_1
    long tu;
    int tuflag;
#endif

    pname = strrchr(av[0], '/');
    pname = pname ? ++pname : av[0];

    if((ac == 1) || (*av[1] == 'l'))
      listonly = 1;
    
    if (ac == 1) {
	listuser("-");
	exit(EX_OK);
    }

    if (ac >= 3) { 
      user = av[2];
#ifdef DBF_1
      if (!strcmp(user, lmt_name)) {
	printf("User name `%s' is reserved.\n", user);
	exit(EX_NOUSER);
      }
#endif      
    }

    switch (*av[1]) {
    case 'a':
    case 'A':
    case 'c':
    case 'C':
#ifdef DBF_1
    case 's':
    case 'S':
	if (ac != 7 && ac != 8)
#else
	if (ac != 4)
#endif
	    usage();
#ifdef DBF_1
        if(ac == 8)
         {
          tuflag = 1;
          tu = atol(av[7]) * 60;
         }
        else if(*(av[1] + 1) == 'z' || *(av[1] + 1) == 'Z')
         {
          tuflag = -1;
          tu = 0;
         }
        else
         {
          tuflag = 0;
          tu = 0;
         }
         
	adduser(user, tolower(*av[1]),
	atol(av[3]) * 60, atol(av[4]) * 60, atol(av[5]) * 60, atol(av[6]) * 60,
	     tu, tuflag);
#else
	adduser(user, tolower(*av[1]), atoi(av[3]) * 60);
#endif
	break;
	
    case 'm':
    case 'M':
#ifdef DBF_1
	if (ac != 7 && ac != 8)
#else
	if (ac != 4)
#endif
	    usage();
#ifdef DBF_1
        if(ac == 8)
         {
          tuflag = 1;
          tu = atol(av[7]) * 60;
         }
        else if(*(av[1] + 1) == 'z' || *(av[1] + 1) == 'Z')
         {
          tuflag = -1;
          tu = 0;
         }
        else
         {
          tuflag = 0;
          tu = 0;
         }
         
	moduser(user,
	atol(av[3]) * 60, atol(av[4]) * 60, atol(av[5]) * 60, atol(av[6]) * 60,
	     tu, tuflag);
#else
	moduser(user, atoi(av[3]) * 60);
#endif
	break;
	

    case 'd':
    case 'D':
	if (ac != 3)
	    usage();
	deluser(user);
	break;
    case 'l':
    case 'L':
	if (ac != 3)
	    usage();
#ifdef DBF_1
        if(*(av[1] + 1) == 'z' || *(av[1] + 1) == 'Z')
          ph = 0;
#endif
	listuser(user);
	break;
#ifdef DBF_1	
    case 't':
    case 'T':
	if (ac > 3)
	    usage();
	lmtset(user);
	renew_day_limit();
	break;
#endif
    default:
	usage();
	break;
    }
    if(db)
      dbm_close(db);
    exit(EX_OK);
}
