/* 
   Copyright (c) 1995-1996 by Cisco systems, Inc.

   Permission to use, copy, modify, and distribute this software for
   any purpose and without fee is hereby granted, provided that this
   copyright and permission notice appear on all copies of the
   software and supporting documentation, the name of Cisco Systems,
   Inc. not be used in advertising or publicity pertaining to
   distribution of the program without specific prior permission, and
   notice be given in supporting documentation that modification,
   copying and distribution is by permission of Cisco Systems, Inc.

   Cisco Systems, Inc. makes no representations about the suitability
   of this software for any purpose.  THIS SOFTWARE IS PROVIDED ``AS
   IS'' AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING,
   WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
   FITNESS FOR A PARTICULAR PURPOSE.
*/

#include "tac_plus.h"

static int acctfd = 0;

/* Make a acct entry into the accounting file for accounting. 
   Return 1 on error  */

static int
acct_write(string)
    char *string;
{
    if (write(acctfd, string, strlen(string)) != strlen(string)) {
	report(LOG_ERR, "%s: couldn't write acct file %s %s",
	       session.peer,
	       session.acctfile, sys_errlist[errno]);
	return(1);
    }
    
    if (debug & DEBUG_ACCT_FLAG)
	report(LOG_DEBUG, "'%s'", string);

    return(0);
}

/* Write a string or "unknown" into the accounting file.
   Return 1 on error  */
static int
acct_write_field(string)
    char *string;
{
    if (string && string[0]) {
	if (acct_write(string))
	    return(1);
    } else {
	if (acct_write("unknown"))
	    return(1);
    }
    return(0);
}

int
do_acct(rec)
struct acct_rec *rec;
{
    int i, errors;
    time_t t = time(NULL);

#ifndef NO_BWM_DATE
    char atime[256];
#endif

#ifdef UTMP    
    struct utmp u;
#endif

    if (!acctfd) {
	acctfd = open(session.acctfile, O_CREAT | O_WRONLY | O_APPEND, 0666);
	if (acctfd < 0) {
	    report(LOG_ERR, "Can't open acct file %s -- %s",
		   session.acctfile, sys_errlist[errno]);
	    return(1);
	}
    }
    if (!tac_lockfd(session.acctfile, acctfd)) {
	rec->admin_msg = tac_strdup("Cannot lock log file");
	report(LOG_ERR, "%s: Cannot lock %s", 
	       session.peer, session.acctfile);
	return(1);
    }

    errors = 0;

#ifdef UTMP
    bzero(&u, sizeof(u));
    u.ut_time = t;
    strncpy(u.ut_host, rec->identity->NAS_name, UT_HOSTSIZE);
/*    u.ut_host[UT_HOSTSIZE-1] = '\0'; */
    strncpy(u.ut_line, rec->identity->NAS_port, UT_LINESIZE);
/*    u.ut_line[UT_LINESIZE-1] = '\0'; */
#endif

#ifdef NO_BWM_DATE
    {
     char *ct = ctime(&t);
     ct[24] = '\0';
     errors += acct_write(ct);
    }
#else
    strftime(atime, sizeof(atime), "%Y/%m/%d %H:%M:%S", localtime(&t));
    errors += acct_write(atime);
#endif

    errors += acct_write("\t");

#ifndef NO_BWM_DATE
    sprintf(atime, "%u", t);
    errors += acct_write(atime);
    errors += acct_write("\t");
#endif

    errors += acct_write_field(rec->identity->NAS_name);
    errors += acct_write("\t");

    errors += acct_write_field(rec->identity->username);
    errors += acct_write("\t");

    errors += acct_write_field(rec->identity->NAS_port);
    errors += acct_write("\t");

    errors += acct_write_field(rec->identity->NAC_address);
    errors += acct_write("\t");

    switch(rec->acct_type) {
    case ACCT_TYPE_UPDATE:
	errors += acct_write("update\t");
	break;
    case ACCT_TYPE_START:
	errors += acct_write("start\t");
#ifdef UTMP
        if(rec->authen_service == TAC_PLUS_AUTHEN_SVC_LOGIN ||
           rec->authen_service == TAC_PLUS_AUTHEN_SVC_PPP)
         {
          strncpy(u.ut_name, rec->identity->username, UT_NAMESIZE);
/*          u.ut_name[UT_NAMESIZE-1] = '\0'; */

#ifdef ALIASES
          loginal(&u, rec->identity->NAS_alias, UTMP_LOG_MODE);
#else
          login(&u);
#endif
         }
#endif        
	break;
    case ACCT_TYPE_STOP:
	errors += acct_write("stop\t");
#ifdef UTMP
        if(rec->authen_service == TAC_PLUS_AUTHEN_SVC_LOGIN ||
           rec->authen_service == TAC_PLUS_AUTHEN_SVC_PPP)
         {
#ifdef ALIASES
          loginal(&u, rec->identity->NAS_alias, UTMP_LOG_MODE);
#else
          login(&u);
#endif
         }
#endif        
	break;
    default:
	errors += acct_write("unknown\t");
	break;
    }

    for (i=0; i < rec->num_args; i++) {
	errors += acct_write(rec->args[i]);
	if (i < (rec->num_args-1)) 
	    errors += acct_write("\t");
    }
/* **** pasa's debug code
     sprintf(atime, "\txmethod=%d\txtype=%d\txservice=%d",
              rec->authen_method, rec->authen_type, rec->authen_service);
     errors += acct_write(atime);
 **** */
    errors += acct_write("\n");

    close(acctfd);
    acctfd = 0;

    if (errors) {
	return(1);
    }
    return (0);
}

